@@ -9,7 +9,7 @@
 ## need to store the number of rows (nrows).
 setClass("DataFrame",
          representation(
-                        rownames = "characterORNULL",
+                        rownames = "character_OR_NULL",
                         nrows = "integer"
                         ),
          prototype(rownames = NULL,
@@ -205,6 +205,15 @@ DataFrame <- function(..., row.names = NULL, check.names = TRUE)
 ### Subsetting.
 ###
 
+setMethod("[[", "DataFrame", function(x, i, j, ...)
+{
+    if (!missing(j)) {
+        x[[j, ...]][[i]]
+    } else {
+        callNextMethod()
+    }
+})
+
 setReplaceMethod("[[", "DataFrame",
                  function(x, i, j,..., value)
                  {
@@ -277,7 +286,7 @@ setMethod("[", "DataFrame",
             x <- initialize(x, listData=new_listData,
                                elementMetadata=new_mcols)
             if (anyDuplicated(names(x)))
-                names(x) <- make.names(names(x))
+                names(x) <- make.unique(names(x))
             if (list_style_subsetting)
                 return(x)
         }
@@ -636,7 +645,7 @@ setAs("AsIs", "DataFrame",
 
 setAs("ANY", "AsIs", function(from) I(from))
 
-setAs("ANY", "DataTableORNULL", function(from) as(from, "DataFrame"))
+setAs("ANY", "DataTable_OR_NULL", function(from) as(from, "DataFrame"))
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Combining.
