@@ -51,12 +51,6 @@ setMethod("elementNROWS", "List",
     }
 )
 
-### In BioC 3.3 elementLengths() was renamed elementNROWS() (the old name was
-### clearly a misnomer). For backward compatibility the old name was kept
-### around, deprecated, and became an "alias" for elementNROWS().
-setGeneric("elementLengths", function(x) standardGeneric("elementLengths"))
-setMethod("elementLengths", "ANY", function(x) .Defunct("elementNROWS"))
-
 setGeneric("isEmpty", function(x) standardGeneric("isEmpty"))
 setMethod("isEmpty", "ANY",
           function(x)
