@@ -3,38 +3,32 @@
 ### -------------------------------------------------------------------------
 ###
 
+
 setClass("Rle",
-         representation(values = "vectorORfactor",
-                        lengths = "integer"),
-         prototype = prototype(values = logical()),
-         contains = "Vector",
-         validity = function(object)
-         {
-             msg <- NULL
-             run_values <- runValue(object)
-             run_lengths <- runLength(object)
-             if (length(run_values) != length(run_lengths))
-                 msg <- c(msg, "run values and run lengths must have the same length")
-             if (!all(run_lengths > 0L))
-                 msg <- c(msg, "all run lengths must be positive")
-             ## TODO: Fix the following test.
-             #if (length(run_lengths) >= 2 && is.atomic(run_values)
-             #      && any(run_values[-1L] == run_values[-length(run_values)]))
-             #    msg <- c(msg, "consecutive runs must have different values")
-             if (is.null(msg)) TRUE else msg
-         })
+    contains="Vector",
+    representation(
+        values="vector_OR_factor",
+        lengths="integer_OR_Linteger"
+    ),
+    prototype(
+        values=logical(0),
+        lengths=integer(0)
+    )
+)
 
  
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Getters
 ###
 
+setMethod("length", "Rle",
+    function(x) as.double(.Call2("Rle_length", x, PACKAGE="S4Vectors"))
+)
+
 setGeneric("runLength", signature = "x",
            function(x) standardGeneric("runLength"))
 setMethod("runLength", "Rle", function(x) x@lengths)
  
-setMethod("length", "Rle", function(x) sum(runLength(x)))
-
 setGeneric("runValue", signature = "x",
            function(x) standardGeneric("runValue"))
 setMethod("runValue", "Rle", function(x) x@values)
@@ -47,32 +41,59 @@ setMethod("end", "Rle", function(x) .Call2("Rle_end", x, PACKAGE="S4Vectors"))
 setMethod("width", "Rle", function(x) runLength(x))
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Validity
+###
+
+.valid_Rle <- function(x)
+{
+    msg <- NULL
+    msg <- c(msg, .Call2("Rle_valid", x, PACKAGE="S4Vectors"))
+    ## Too expensive so commented out for now. Maybe do this in C?
+    #run_values <- runValues(x)
+    #if (length(run_values) >= 2 && is.atomic(run_values) &&
+    #    any(run_values[-1L] == run_values[-length(run_values)]))
+    #    msg <- c(msg, "consecutive runs must have different values")
+    msg
+}
+
+setValidity2("Rle", .valid_Rle)
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Constructor
 ###
 
 ### Low-level constructor.
-new_Rle <- function(values=logical(0), lengths=integer(0), check=TRUE)
+new_Rle <- function(values=logical(0), lengths=NULL)
 {
-    stopifnot(is(values, "vectorORfactor"))
-    stopifnot(is.numeric(lengths))
-    if (!is.integer(lengths))
-        lengths <- as.integer(lengths)
-    if (!isTRUEorFALSE(check))
-        stop("'check' must be TRUE or FALSE")
-    .Call2("Rle_constructor", values, lengths, check, 0L, PACKAGE="S4Vectors")
+    stopifnot(is(values, "vector_OR_factor"))
+    if (!is.null(lengths)) {
+        if (!(is.numeric(lengths) || is.Linteger(lengths)))
+            stop("'lengths' must be NULL or a numeric or Linteger vector")
+        if (anyNA(lengths))
+            stop("'lengths' cannot contain NAs")
+        if (is.double(lengths)) {
+            suppressWarnings(lengths <- as.Linteger(lengths))
+            if (anyNA(lengths))
+                stop("Rle vector is too long")
+        }
+        if (length(lengths) == 1L)
+            lengths <- rep.int(lengths, length(values))
+    }
+    .Call2("Rle_constructor", values, lengths, PACKAGE="S4Vectors")
 }
 
 setGeneric("Rle", signature="values",
-    function(values=logical(0), lengths=integer(0)) standardGeneric("Rle")
+    function(values=logical(0), lengths=NULL) standardGeneric("Rle")
 )
 
 setMethod("Rle", "ANY",
-    function(values=logical(0), lengths=integer(0)) new_Rle(values, lengths)
+    function(values=logical(0), lengths=NULL) new_Rle(values, lengths)
 )
 
 setMethod("Rle", "Rle",
-    function(values=logical(0), lengths=integer(0))
+    function(values=logical(0), lengths=NULL)
     {
         if (!missing(lengths))
             stop(wmsg("'lengths' cannot be supplied when calling Rle() ",
@@ -167,34 +188,32 @@ getStartEndRunAndOffset <- function(x, start, end) {
 ### Note that they drop the metadata columns!
 ###
 
+### TODO: Support NAs in 'pos'.
+extract_positions_from_Rle <- function(x, pos, method=0L, decoded=FALSE)
+{
+    if (!is.integer(pos))
+        stop("'pos' must be an integer vector")
+    if (!isTRUEorFALSE(decoded))
+        stop("'decoded' must be TRUE or FALSE")
+    #ans <- .Call2("Rle_extract_positions", x, pos, method, PACKAGE="S4Vectors")
+    mapped_pos <- map_positions_to_runs(runLength(x), pos, method=method)
+    ans <- runValue(x)[mapped_pos]
+    if (decoded)
+        return(ans)
+    as(Rle(ans), class(x))  # so the function is an endomorphism
+}
+
 extract_range_from_Rle <- function(x, start, end)
 {
     ans <- .Call2("Rle_extract_range", x, start, end, PACKAGE="S4Vectors")
     as(ans, class(x))  # so the function is an endomorphism
 }
 
-.normarg_method <- function(method)
-{
-    if (!(isSingleNumber(method) && method >= 0 && method <= 3))
-        stop("'method' must be a single integer between 0 and 3")
-    if (!is.integer(method))
-        method <- as.integer(method)
-    method
-}
-
-### Used in GenomicRanges.
-map_ranges_to_runs <- function(run_lens, start, width, method=0L)
-{
-    method <- .normarg_method(method)
-    .Call2("ranges_to_runs_mapper", run_lens, start, width, method,
-                                    PACKAGE="S4Vectors")
-}
-
 ### NOT exported but used in IRanges package (by "extractROWS" method with
 ### signature Rle,RangesNSBS).
 extract_ranges_from_Rle <- function(x, start, width, method=0L, as.list=FALSE)
 {
-    method <- .normarg_method(method)
+    method <- normarg_method(method)
     if (!isTRUEorFALSE(as.list))
         stop("'as.list' must be TRUE or FALSE")
     ans <- .Call2("Rle_extract_ranges", x, start, width, method, as.list,
@@ -209,20 +228,6 @@ extract_ranges_from_Rle <- function(x, start, width, method=0L, as.list=FALSE)
     lapply(ans, as, x_class)
 }
 
-### TODO: extract_positions_from_Rle() should call its own .Call entry point.
-### This entry point would be a simpler version of .Call entry point
-### "Rle_extract_ranges" that takes 1 integer vector instead of 2 and supports
-### NAs.
-extract_positions_from_Rle <- function(x, i)
-{
-    if (!is.integer(i))
-        stop("'i' must be an integer vector")
-    ## NAs not supported for now but will be soon (see TODO above).
-    if (anyNA(i))
-        stop("numeric subscript cannot contain NAs yet when subsetting an Rle")
-    extract_ranges_from_Rle(x, i, rep.int(1L, length(i)))
-}
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Subsetting
@@ -622,7 +627,7 @@ setMethod("append", c("vector", "Rle"),
 
 setMethod("%in%", "Rle",
           function(x, table)
-              new_Rle(runValue(x) %in% table, runLength(x), check=FALSE))
+              new_Rle(runValue(x) %in% table, runLength(x)))
 
 setGeneric("findRun", signature = "vec",
            function(x, vec) standardGeneric("findRun"))
@@ -637,7 +642,7 @@ setMethod("findRun", signature = c(vec = "Rle"),
 
 setMethod("is.na", "Rle",
           function(x)
-              new_Rle(is.na(runValue(x)), runLength(x), check=FALSE))
+              new_Rle(is.na(runValue(x)), runLength(x)))
 
 setMethod("anyNA", "Rle",
           function(x)
@@ -688,7 +693,8 @@ setMethod("match", c("Rle", "Rle"),
 
 ### FIXME: Remove in R 3.3
 setMethod("order", "Rle",
-    function(..., na.last=TRUE, decreasing=FALSE, method=c("shell", "radix"))
+    function(..., na.last=TRUE, decreasing=FALSE,
+                  method=c("auto", "shell", "radix"))
     {
         args <- lapply(unname(list(...)), decodeRle)
         do.call(order, c(args, list(na.last=na.last,
@@ -704,7 +710,7 @@ setMethod("order", "Rle",
             x <- x[!is.na(x)]
     }
     ord <- base::order(runValue(x), na.last=na.last, decreasing=decreasing)
-    new_Rle(runValue(x)[ord], runLength(x)[ord], check=FALSE)
+    new_Rle(runValue(x)[ord], runLength(x)[ord])
 }
 setMethod("sort", "Rle", .sort.Rle)
 
@@ -856,8 +862,8 @@ setMethod("show", "Rle",
               nr <- nrun(object)
               halfWidth <- getOption("width") %/% 2L
               cat(classNameForDisplay(runValue(object)),
-                  "-Rle of length ", lo, " with ", nr,
-                  ifelse(nr == 1, " run\n", " runs\n"), sep = "")
+                  "-Rle of length ", as.character(as.Linteger(lo)),
+                  " with ", nr, ifelse(nr == 1, " run\n", " runs\n"), sep = "")
               first <- max(1L, halfWidth)
               showMatrix <-
                 rbind(as.character(head(runLength(object), first)),
