@@ -7,13 +7,13 @@
 ###
 
 
-setClassUnion("DataTableORNULL", c("DataTable", "NULL"))
+setClassUnion("DataTable_OR_NULL", c("DataTable", "NULL"))
 
 setClass("Vector",
     contains="Annotated",
     representation(
         "VIRTUAL",
-        elementMetadata="DataTableORNULL"
+        elementMetadata="DataTable_OR_NULL"
     )
 )
 
@@ -109,6 +109,7 @@ setMethod("values", "Vector", function(x, ...) elementMetadata(x, ...))
 
 setMethod("anyNA", "Vector", function(x, recursive=FALSE) any(is.na(x)))
 
+setMethod("is.na", "Vector", function(x) rep(FALSE, length(x)))
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Validity.
@@ -117,8 +118,8 @@ setMethod("anyNA", "Vector", function(x, recursive=FALSE) any(is.na(x)))
 .valid.Vector.length <- function(x)
 {
     x_len <- length(x)
-    if (!isSingleInteger(x_len) || x_len < 0L)
-        return("'length(x)' must be a single non-negative integer")
+    if (!isSingleNumber(x_len) || x_len < 0L)
+        return("'length(x)' must be a single non-negative number")
     if (!is.null(attributes(x_len)))
         return("'length(x)' must be a single integer with no attributes")
     NULL
@@ -173,7 +174,7 @@ setMethod("anyNA", "Vector", function(x, recursive=FALSE) any(is.na(x)))
 .valid.Vector.mcols <- function(x)
 {
     x_mcols <- mcols(x)
-    if (!is(x_mcols, "DataTableORNULL"))
+    if (!is(x_mcols, "DataTable_OR_NULL"))
         return("'mcols(x)' must be a DataTable object or NULL")
     if (is.null(x_mcols))
         return(NULL)
@@ -262,13 +263,18 @@ setMethod("as.matrix", "Vector", function(x) {
               as.matrix(as.vector(x))
           })
 
+classNamespace <- function(x) {
+    pkg <- packageSlot(class(x))
+    pvnEnv <- .GlobalEnv
+    if (!is.null(pkg)) {
+        pvnEnv <- getNamespace(pkg)
+    }
+    pvnEnv
+}
+
 makeFixedColumnEnv <- function(x, parent, tform = identity) {
   env <- new.env(parent=parent)
-  pkg <- packageSlot(class(x))
-  pvnEnv <- .GlobalEnv
-  if (!is.null(pkg)) {
-      pvnEnv <- getNamespace(pkg)
-  }
+  pvnEnv <- classNamespace(x)
   lapply(c("names", parallelVectorNames(x)), function(nm) {
     accessor <- get(nm, pvnEnv, mode="function")
     makeActiveBinding(nm, function() {
@@ -372,16 +378,7 @@ setMethod("rename", "Vector", .renameVector)
 ### "extractROWS" and "replaceROWS" method for his/her objects.
 ###
 
-setMethod("[", "Vector",
-    function(x, i, j, ..., drop=TRUE)
-    {
-        if (!missing(j) || length(list(...)) > 0L)
-            stop("invalid subsetting")
-        if (missing(i))
-            return(x)
-        extractROWS(x, i)
-    }
-)
+setMethod("[", "Vector", subset_along_ROWS)
 
 ### We provide a default "extractROWS" method for Vector objects that only
 ### subsets the individual parallel slots. That should be enough for most
@@ -482,120 +479,19 @@ Vector_window <- function(x, start=NA, end=NA, width=NA)
 setMethod("window", "Vector", Vector_window)
 
 ### S3/S4 combo for head.Vector
-head.Vector <- function(x, n=6L, ...) head(x, n, ...)
-Vector_head <- function(x, n=6L)
-{
-    if (!isSingleNumber(n))
-        stop("'n' must be a single integer")
-    if (!is.integer(n))
-        n <- as.integer(n)
-    x_NROW <- NROW(x)
-    if (n >= 0L) {
-        n <- min(x_NROW, n)
-    } else {
-        n <- max(0L, x_NROW + n)
-    }
-    window(x, start=1L, width=n)
-}
-setMethod("head", "Vector", Vector_head)
+head.Vector <- function(x, ...) head(x, ...)
+setMethod("head", "Vector", head_along_ROWS)
 
 ## S3/S4 combo for tail.Vector
-tail.Vector <- function(x, n=6L, ...) tail(x, n, ...)
-Vector_tail <- function(x, n=6L)
-{
-    if (!isSingleNumber(n))
-        stop("'n' must be a single integer")
-    if (!is.integer(n))
-        n <- as.integer(n)
-    x_NROW <- NROW(x)
-    if (n >= 0L) {
-        n <- min(x_NROW, n)
-    } else {
-        n <- max(0L, x_NROW + n)
-    }
-    window(x, end=x_NROW, width=n)
-}
-setMethod("tail", "Vector", Vector_tail)
+tail.Vector <- function(x, ...) tail(x, ...)
+setMethod("tail", "Vector", tail_along_ROWS)
+
+setMethod("rep.int", "Vector", rep.int_along_ROWS)
 
 ## NOT exported.
 revROWS <- function(x) extractROWS(x, rev(seq_len(NROW(x))))
 
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Internal helpers used by the "show" method of various Vector subclasses.
-###
-
-setGeneric("classNameForDisplay",
-    function(x) standardGeneric("classNameForDisplay"))
-
-setMethod("classNameForDisplay", "ANY",
-   function(x)
-   {
-       ## Selecting the 1st element guarantees that we return a single string
-       ## (e.g. on an ordered factor, class(x) returns a character vector of
-       ## length 2).
-       class(x)[1L]
-   }
-)
-
-.drop_AsIs <- function(x)
-{
-    #x_class <- class(x)
-    #if (x_class[[1L]] == "AsIs")
-    #    class(x) <- x_class[-1L]
-
-    ## Simpler, and probably more robust, than the above.
-    class(x) <- setdiff(class(x), "AsIs")
-    x
-}
-
-setMethod("classNameForDisplay", "AsIs",
-    function(x) classNameForDisplay(.drop_AsIs(x))
-)
-
-
-### All "showAsCell" methods should return a character vector.
-setGeneric("showAsCell",
-    function(object) standardGeneric("showAsCell")
-)
-
-setMethod("showAsCell", "AsIs",
-    function(object) showAsCell(.drop_AsIs(object))
-)
-
-setMethod("showAsCell", "ANY", function(object) {
-  if (NROW(object) == 0L)
-    return(character(0L))
-  if (length(dim(object)) > 2)
-    dim(object) <- c(nrow(object), prod(tail(dim(object), -1)))
-  if (NCOL(object) > 1) {
-    df <- as.data.frame(object[, head(seq_len(ncol(object)), 3), drop = FALSE])
-    attempt <- do.call(paste, c(df, sep=":"))
-    if (ncol(object) > 3L)
-      attempt <- paste0(attempt, ":...")
-    attempt
-  } else if (NCOL(object) == 0L) {
-    rep.int("", NROW(object))
-  } else if (is.list(object) || is(object, "List")) {
-    vapply(object, function(x) {
-      str <- paste(head(x, 3L), collapse = ",") 
-      if (length(x) > 3L)
-        str <- paste0(str, ",...")
-      str
-    }, character(1L))
-  } else {
-    attempt <- try(as.vector(object), silent=TRUE)
-    if (is(attempt, "try-error"))
-      rep.int("########", length(object))
-    else attempt
-  }
-})
-
-### Mmmh... these methods don't return a character vector. Is that ok?
-setMethod("showAsCell", "Date", function(object) object)
-setMethod("showAsCell", "POSIXt", function(object) object)
-
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Combining.
 ###
@@ -677,11 +573,26 @@ setMethod("with", "Vector",
             safeEval(substitute(expr), data, parent.frame(), ...)
           })
 
+setReplaceMethod("column", "Vector", function(x, name, value) {
+    if (name %in% parallelVectorNames(x)) {
+        setter <- get(paste0(name, "<-"), classNamespace(x), mode="function")
+        setter(x, value=value)
+    } else {
+        mcols(x)[[name]] <- value
+        x
+    }
+})
+
+transform.Vector <- transformColumns
+
+setMethod("transform", "Vector", transform.Vector)
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Utilities.
 ###
 
 setGeneric("expand.grid", signature="...")
+BiocGenerics:::apply_hotfix73465(getGeneric("expand.grid"))
 
 setMethod("expand.grid", "Vector",
           function(..., KEEP.OUT.ATTRS = TRUE, stringsAsFactors = TRUE) {
