@@ -458,15 +458,73 @@ setGeneric("replaceROWS", signature="x",
 
 setMethod("extractROWS", c("ANY", "ANY"), .extractROWSWithBracket)
 
-setMethod("extractROWS", c("vectorORfactor", "RangeNSBS"),
+### NOT exported but used in IRanges package (by "extractROWS" method with
+### signature vector_OR_factor,RangesNSBS).
+extract_ranges_from_vector_OR_factor <- function(x, start, width)
+{
+    .Call2("vector_OR_factor_extract_ranges", x, start, width,
+                                              PACKAGE="S4Vectors")
+}
+
+setMethod("extractROWS", c("vector_OR_factor", "RangeNSBS"),
     function(x, i)
     {
         start <- i@subscript[[1L]]
         width <- i@subscript[[2L]] - start + 1L
-        extract_ranges_from_vectorORfactor(x, start, width)
+        extract_ranges_from_vector_OR_factor(x, start, width)
     }
 )
 
+### NOT exported but will be used in IRanges package (by "extractROWS" method
+### with signature Linteger,RangesNSBS).
+extract_ranges_from_Linteger <- function(x, start, width)
+{
+    start <- (start - 1L) * BYTES_PER_LINTEGER + 1L
+    width <- width * BYTES_PER_LINTEGER
+    x@bytes <- extract_ranges_from_vector_OR_factor(x@bytes, start, width)
+    x
+}
+
+setMethod("extractROWS", c("Linteger", "RangeNSBS"),
+    function(x, i)
+    {
+        start <- i@subscript[[1L]]
+        width <- i@subscript[[2L]] - start + 1L
+        extract_ranges_from_Linteger(x, start, width)
+    }
+)
+
+setMethod("extractROWS", c("Linteger", "NSBS"),
+    function(x, i)
+    {
+        start <- as.integer(i)
+        width <- rep.int(1L, length(start))
+        extract_ranges_from_Linteger(x, start, width)
+    }
+)
+
+setMethod("extractROWS", c("Linteger", "ANY"),
+    function (x, i)
+    {
+        ## We don't support NAs in the subscript yet.
+        #i <- normalizeSingleBracketSubscript(i, x, allow.NAs=TRUE,
+        #                                           as.NSBS=TRUE)
+        i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
+        callGeneric()
+    }
+)
+
+subset_along_ROWS <- function(x, i, j, ..., drop=TRUE)
+{
+    if (!missing(j) || length(list(...)) > 0L)
+        stop("invalid subsetting")
+    if (missing(i))
+        return(x)
+    extractROWS(x, i)
+}
+
+setMethod("[", "Linteger", subset_along_ROWS)
+
 setMethod("replaceROWS", "ANY", .replaceROWSWithBracket)
 
 
@@ -558,3 +616,79 @@ setMethod("getListElement", "list",
     }
 )
 
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### window(), head(), tail(), rep.int()
+###
+
+### S3/S4 combo for window.Linteger
+window_along_ROWS <- function(x, start=NA, end=NA, width=NA)
+{
+    i <- RangeNSBS(x, start=start, end=end, width=width)
+    extractROWS(x, i)
+}
+window.Linteger <- function(x, ...) window_along_ROWS(x, ...)
+setMethod("window", "Linteger", window.Linteger)
+
+### S3/S4 combo for head.Linteger
+head_along_ROWS <- function(x, n=6L)
+{
+    if (!isSingleNumber(n))
+        stop("'n' must be a single integer")
+    if (!is.integer(n))
+        n <- as.integer(n)
+    x_NROW <- NROW(x)
+    if (n >= 0L) {
+        n <- min(x_NROW, n)
+    } else {
+        n <- max(0L, x_NROW + n)
+    }
+    window(x, start=1L, width=n)
+}
+head.Linteger <- function(x, ...) head_along_ROWS(x, ...)
+setMethod("head", "Linteger", head.Linteger)
+
+### S3/S4 combo for tail.Linteger
+tail_along_ROWS <- function(x, n=6L)
+{
+    if (!isSingleNumber(n))
+        stop("'n' must be a single integer")
+    if (!is.integer(n))
+        n <- as.integer(n)
+    x_NROW <- NROW(x)
+    if (n >= 0L) {
+        n <- min(x_NROW, n)
+    } else {
+        n <- max(0L, x_NROW + n)
+    }
+    window(x, end=x_NROW, width=n)
+}
+tail.Linteger <- function(x, ...) tail_along_ROWS(x, ...)
+setMethod("tail", "Linteger", tail.Linteger)
+
+rep.int_along_ROWS <- function(x, times)
+{
+    x_len <- length(x)
+    if (!(is.numeric(times) || is.Linteger(times)))
+        stop("'times' must be a numeric or Linteger vector")
+    times_len <- length(times)
+    if (times_len == 1L) {
+        if (times == 1L)
+            return(x)
+        if (times == 0L)
+            return(extractROWS(x, integer(0)))
+    }
+    if (times_len == x_len) {
+        i <- Rle(seq_len(x_len), times)
+    } else if (times_len == 1L) {
+        if (is.Linteger(times))
+            times <- as.double(times)
+        i <- IRanges::IRanges(rep.int(1L, times), rep.int(x_len, times))
+    } else {
+        stop("invalid 'times' value")
+    }
+    extractROWS(x, i)
+}
+
+setMethod("rep.int", "Linteger", rep.int_along_ROWS)
+
