@@ -4,9 +4,32 @@
  ****************************************************************************/
 #include "S4Vectors.h"
 #include <stdlib.h>  /* for malloc, free, realloc */
+#include <limits.h>  /* for INT_MAX */
 
-#define MAX_BUFLENGTH_INC (32 * 1024 * 1024)
-#define MAX_BUFLENGTH (63 * MAX_BUFLENGTH_INC)
+
+#define MAX_BUFLENGTH_INC 33554432ULL  // 2^25
+/* IMPORTANT: Keep MAX_BUFLENGTH <= R_XLEN_T_MAX (i.e. 2^52, see Rinternals.h)
+   otherwise casting a buffer length (size_t) to R_xlen_t will not do the
+   right thing (undefined behavior).
+   For now we set MAX_BUFLENGTH to 4294967296 only (i.e. 2^32). This is big
+   enough to support buffers of the length of the human genome. */
+#define MAX_BUFLENGTH (128ULL * MAX_BUFLENGTH_INC)  // 2^32
+
+/* Guaranteed to return a value > 'buflength', or to raise an error. */
+size_t _increase_buflength(size_t buflength)
+{
+	if (buflength >= MAX_BUFLENGTH)
+		error("S4Vectors internal error in _increase_buflength(): "
+		      "MAX_BUFLENGTH reached");
+	if (buflength == 0)
+		return 128;
+	if (buflength <= MAX_BUFLENGTH_INC)
+		return 2 * buflength;
+	buflength += MAX_BUFLENGTH_INC;
+	if (buflength <= MAX_BUFLENGTH)
+		return buflength;
+	return MAX_BUFLENGTH;
+}
 
 
 /****************************************************************************
@@ -21,64 +44,55 @@ SEXP AEbufs_use_malloc(SEXP x)
 	return R_NilValue;
 }
 
-/* 'nmemb' must be > 0. This is NOT checked! */
-static void *alloc2(int nmemb, size_t size)
+static void *alloc2(size_t nmemb, size_t memb_size)
 {
 	void *ptr;
 
+	if (nmemb > MAX_BUFLENGTH)
+		error("S4Vectors internal error in alloc2(): "
+		      "buffer is too big");
 	if (use_malloc) {
 		//printf("alloc2: nmemb=%d\n", nmemb);
-		size *= nmemb;
-		ptr = malloc(size);
+		memb_size *= nmemb;
+		ptr = malloc(memb_size);
 		if (ptr == NULL)
 			error("S4Vectors internal error in alloc2(): "
 			      "cannot allocate memory");
 	} else {
-		ptr = (void *) R_alloc(size, nmemb);
+		ptr = (void *) R_alloc(nmemb, (int) memb_size);
 	}
 	return ptr;
 }
 
 /* 'new_nmemb' must be > 'old_nmemb'. */
-static void *realloc2(void *ptr, int new_nmemb, int old_nmemb, size_t size)
+static void *realloc2(void *ptr, size_t old_nmemb, size_t new_nmemb,
+		      size_t memb_size)
 {
 	void *new_ptr;
 
+	if (new_nmemb > MAX_BUFLENGTH)
+		error("S4Vectors internal error in realloc2(): "
+		      "buffer is too big");
 	if (new_nmemb <= old_nmemb)
 		error("S4Vectors internal error in realloc2(): "
-		      "'new_nmemb' <= 'old_nmemb'");
+		      "'new_nmemb' must be > 'old_nmemb'");
 	if (old_nmemb == 0)
-		return alloc2(new_nmemb, size);
+		return alloc2(new_nmemb, memb_size);
 	if (use_malloc) {
-		//printf("realloc2: new_nmemb=%d old_nmemb=%d\n",
+		//printf("realloc2: new_nmemb=%lu old_nmemb=%lu\n",
 		//       new_nmemb, old_nmemb);
-		size *= new_nmemb;
-		new_ptr = realloc(ptr, size);
+		memb_size *= new_nmemb;
+		new_ptr = realloc(ptr, memb_size);
 		if (new_ptr == NULL)
 			error("S4Vectors internal error in realloc2(): "
 			      "cannot reallocate memory");
 	} else {
-		new_ptr = (void *) R_alloc(size, new_nmemb);
-		memcpy(new_ptr, ptr, size * old_nmemb);
+		new_ptr = (void *) R_alloc(new_nmemb, (int) memb_size);
+		memcpy(new_ptr, ptr, old_nmemb * memb_size);
 	}
 	return new_ptr;
 }
 
-/* Guaranteed to return a new buflength > 'buflength', or to raise an error. */
-int _get_new_buflength(int buflength)
-{
-	if (buflength >= MAX_BUFLENGTH)
-		error("_get_new_buflength(): MAX_BUFLENGTH reached");
-	if (buflength == 0)
-		return 128;
-	if (buflength <= MAX_BUFLENGTH_INC)
-		return 2 * buflength;
-	buflength += MAX_BUFLENGTH_INC;
-	if (buflength <= MAX_BUFLENGTH)
-		return buflength;
-	return MAX_BUFLENGTH;
-}
-
 
 /****************************************************************************
  * IntAE buffers
@@ -88,13 +102,17 @@ int _get_new_buflength(int buflength)
 static IntAE *IntAE_pool[INTAE_POOL_MAXLEN];
 static int IntAE_pool_len = 0;
 
-int _IntAE_get_nelt(const IntAE *ae)
+size_t _IntAE_get_nelt(const IntAE *ae)
 {
 	return ae->_nelt;
 }
 
-int _IntAE_set_nelt(IntAE *ae, int nelt)
+size_t _IntAE_set_nelt(IntAE *ae, size_t nelt)
 {
+	if (nelt > ae->_buflength)
+		error("S4Vectors internal error in _IntAE_set_nelt(): "
+		      "trying to set a nb of buffer elements that exceeds "
+		      "the buffer length");
 	return ae->_nelt = nelt;
 }
 
@@ -114,46 +132,54 @@ static IntAE *new_empty_IntAE()
 
 void _IntAE_set_val(const IntAE *ae, int val)
 {
-	int nelt, i, *elt;
+	size_t ae_nelt, i;
+	int *elt_p;
 
-	nelt = _IntAE_get_nelt(ae);
-	for (i = 0, elt = ae->elts; i < nelt; i++, elt++)
-		*elt = val;
+	ae_nelt = _IntAE_get_nelt(ae);
+	elt_p = ae->elts;
+	for (i = 0; i < ae_nelt; i++)
+		*(elt_p++) = val;
 	return;
 }
 
-static void IntAE_extend(IntAE *ae, int new_buflength)
+static void IntAE_extend(IntAE *ae, size_t new_buflength)
 {
-	int old_buflength;
-
-	old_buflength = ae->_buflength;
-	if (new_buflength == -1)
-		new_buflength = _get_new_buflength(old_buflength);
-	ae->elts = (int *) realloc2(ae->elts, new_buflength,
-				    old_buflength, sizeof(int));
+	ae->elts = (int *) realloc2(ae->elts, ae->_buflength,
+				    new_buflength, sizeof(int));
 	ae->_buflength = new_buflength;
 	return;
 }
 
-void _IntAE_insert_at(IntAE *ae, int at, int val)
+static int IntAE_extend_if_full(IntAE *ae)
 {
-	int nelt, i;
+	if (_IntAE_get_nelt(ae) < ae->_buflength)
+		return 0;
+	IntAE_extend(ae, _increase_buflength(ae->_buflength));
+	return 1;
+}
+
+void _IntAE_insert_at(IntAE *ae, size_t at, int val)
+{
+	size_t ae_nelt, i;
 	int *elt1_p;
 	const int *elt2_p;
 
-	nelt = _IntAE_get_nelt(ae);
-	if (nelt >= ae->_buflength)
-		IntAE_extend(ae, -1);
-	elt1_p = ae->elts + nelt;
+	ae_nelt = _IntAE_get_nelt(ae);
+	if (at > ae_nelt)
+		error("S4Vectors internal error in _IntAE_insert_at(): "
+		      "trying to insert a buffer element at an invalid "
+		      "buffer position");
+	IntAE_extend_if_full(ae);
+	elt1_p = ae->elts + ae_nelt;
 	elt2_p = elt1_p - 1;
-	for (i = nelt; i > at; i--)
+	for (i = ae_nelt; i > at; i--)
 		*(elt1_p--) = *(elt2_p--);
 	*elt1_p = val;
-	_IntAE_set_nelt(ae, nelt + 1);
+	_IntAE_set_nelt(ae, ae_nelt + 1);
 	return;
 }
 
-IntAE *_new_IntAE(int buflength, int nelt, int val)
+IntAE *_new_IntAE(size_t buflength, size_t nelt, int val)
 {
 	IntAE *ae;
 
@@ -166,145 +192,156 @@ IntAE *_new_IntAE(int buflength, int nelt, int val)
 	return ae;
 }
 
-void _IntAE_append(IntAE *ae, const int *newvals, int nnewval)
+void _IntAE_append(IntAE *ae, const int *newvals, size_t nnewval)
 {
-	int new_nelt, *dest;
+	size_t ae_nelt, new_nelt;
+	int *dest;
 
-	new_nelt = _IntAE_get_nelt(ae) + nnewval;
+	ae_nelt = _IntAE_get_nelt(ae);
+	new_nelt = ae_nelt + nnewval;
 	if (new_nelt > ae->_buflength)
 		IntAE_extend(ae, new_nelt);
-	dest = ae->elts + _IntAE_get_nelt(ae);
+	dest = ae->elts + ae_nelt;
 	memcpy(dest, newvals, nnewval * sizeof(int));
 	_IntAE_set_nelt(ae, new_nelt);
 	return;
 }
 
-void _IntAE_delete_at(IntAE *ae, int at)
+/*
+ * Delete 'nelt' elements, starting at position 'at'.
+ * Calling _IntAE_delete_at(x, at, nelt) is equivalent to calling
+ * _IntAE_delete_at(x, at, 1) nelt times.
+ */
+void _IntAE_delete_at(IntAE *ae, size_t at, size_t nelt)
 {
 	int *elt1_p;
 	const int *elt2_p;
-	int nelt0, i2;
+	size_t ae_nelt, i2;
 
+	if (nelt == 0)
+		return;
 	elt1_p = ae->elts + at;
-	elt2_p = elt1_p + 1;
-	nelt0 = _IntAE_get_nelt(ae);
-	for (i2 = at + 1; i2 < nelt0; i2++)
+	elt2_p = elt1_p + nelt;
+	ae_nelt = _IntAE_get_nelt(ae);
+	for (i2 = at + nelt; i2 < ae_nelt; i2++)
 		*(elt1_p++) = *(elt2_p++);
-	_IntAE_set_nelt(ae, nelt0 - 1);
+	_IntAE_set_nelt(ae, ae_nelt - nelt);
 	return;
 }
 
-void _IntAE_shift(const IntAE *ae, int shift)
+void _IntAE_shift(const IntAE *ae, size_t offset, int shift)
 {
-	int nelt, i, *elt;
+	size_t ae_nelt, i;
+	int *elt_p;
 
-	nelt = _IntAE_get_nelt(ae);
-	for (i = 0, elt = ae->elts; i < nelt; i++, elt++)
-		*elt += shift;
+	ae_nelt = _IntAE_get_nelt(ae);
+	elt_p = ae->elts + offset;
+	for (i = offset; i < ae_nelt; i++)
+		*(elt_p++) += shift;
 	return;
 }
 
 /*
- * Left and right IntAE objects must have the same length. This is
- * NOT checked!
+ * Left and right IntAE buffers must have the same length.
  */
 void _IntAE_sum_and_shift(const IntAE *ae1, const IntAE *ae2, int shift)
 {
-	int nelt, i, *elt1, *elt2;
+	size_t ae1_nelt, ae2_nelt, i;
+	int *elt1_p;
+	const int *elt2_p;
 
-	nelt = _IntAE_get_nelt(ae1);
-	for (i = 0, elt1 = ae1->elts, elt2 = ae2->elts;
-	     i < nelt;
-	     i++, elt1++, elt2++)
-		*elt1 += *elt2 + shift;
+	ae1_nelt = _IntAE_get_nelt(ae1);
+	ae2_nelt = _IntAE_get_nelt(ae2);
+	if (ae1_nelt != ae2_nelt)
+		error("S4Vectors internal error in _IntAE_sum_and_shift(): "
+		      "the 2 IntAE buffers to sum must have the same length");
+	elt1_p = ae1->elts;
+	elt2_p = ae2->elts;
+	for (i = 0; i < ae1_nelt; i++)
+		*(elt1_p++) += *(elt2_p++) + shift;
 	return;
 }
 
-void _IntAE_append_shifted_vals(IntAE *ae, const int *newvals,
-		int nnewval, int shift)
+void _IntAE_qsort(const IntAE *ae, size_t offset, int desc)
 {
-	int nelt, new_nelt, i, *elt1;
-	const int *elt2;
+	size_t ae_nelt;
 
-	nelt = _IntAE_get_nelt(ae);
-	new_nelt = nelt + nnewval;
-	if (new_nelt > ae->_buflength)
-		IntAE_extend(ae, new_nelt);
-	for (i = 0, elt1 = ae->elts + nelt, elt2 = newvals;
-	     i < nnewval;
-	     i++, elt1++, elt2++)
-		*elt1 = *elt2 + shift;
-	_IntAE_set_nelt(ae, new_nelt);
+	ae_nelt = _IntAE_get_nelt(ae);
+	if (offset > ae_nelt)
+		error("S4Vectors internal error in _IntAE_qsort(): "
+		      "'offset' must be < nb of elements in buffer");
+	_sort_int_array(ae->elts + offset, ae_nelt - offset, desc);
 	return;
 }
 
-void _IntAE_qsort(const IntAE *ae, int desc)
+/*
+ * Delete repeated elements i.e. same semantic as 'uniq' command in Unix.
+ * To get the R unique() behavior (modulo re-ordering of the elements), call
+ * _IntAE_qsort() first.
+ */
+void _IntAE_uniq(IntAE *ae, size_t offset)
 {
-	_sort_int_array(ae->elts, _IntAE_get_nelt(ae), desc);
-	return;
-}
+	size_t ae_nelt, i2;
+	int *elt1_p;
+	const int *elt2_p;
 
-void _IntAE_delete_adjdups(IntAE *ae)
-{
-	int nelt, *elt1;
-	const int *elt2;
-	int i2;
-
-	nelt = _IntAE_get_nelt(ae);
-	if (nelt <= 1)
+	ae_nelt = _IntAE_get_nelt(ae);
+	if (offset > ae_nelt)
+		error("S4Vectors internal error in _IntAE_uniq(): "
+		      "'offset' must be < nb of elements in buffer");
+	if (ae_nelt - offset <= 1)
 		return;
-	elt1 = ae->elts;
-	elt2 = elt1 + 1;
-	for (i2 = 1; i2 < nelt; i2++) {
-		if (*elt2 != *elt1) {
-			elt1++;
-			*elt1 = *elt2;
-		}
-		elt2++;
+	elt1_p = ae->elts + offset;
+	elt2_p = elt1_p + 1;
+	for (i2 = offset + 1; i2 < ae_nelt; i2++) {
+		if (*elt2_p != *elt1_p)
+			*(++elt1_p) = *elt2_p;
+		elt2_p++;
 	}
-	_IntAE_set_nelt(ae, elt1 - ae->elts + 1);
+	_IntAE_set_nelt(ae, elt1_p - ae->elts + 1);
 	return;
 }
 
 SEXP _new_INTEGER_from_IntAE(const IntAE *ae)
 {
-	int nelt;
+	size_t ae_nelt;
 	SEXP ans;
 
-	nelt = _IntAE_get_nelt(ae);
-	PROTECT(ans = NEW_INTEGER(nelt));
-	memcpy(INTEGER(ans), ae->elts, sizeof(int) * nelt);
+	ae_nelt = _IntAE_get_nelt(ae);
+	/* ae_nelt <= R_XLEN_T_MAX so casting is safe. */
+	PROTECT(ans = NEW_INTEGER((R_xlen_t) ae_nelt));
+	memcpy(INTEGER(ans), ae->elts, ae_nelt * sizeof(int));
 	UNPROTECT(1);
 	return ans;
 }
 
-static void copy_INTEGER_to_IntAE(SEXP x, IntAE *ae)
-{
-	_IntAE_set_nelt(ae, LENGTH(x));
-	memcpy(ae->elts, INTEGER(x), sizeof(int) * LENGTH(x));
-	return;
-}
-
 IntAE *_new_IntAE_from_INTEGER(SEXP x)
 {
+	size_t x_len;
 	IntAE *ae;
 
-	ae = _new_IntAE(LENGTH(x), 0, 0);
-	copy_INTEGER_to_IntAE(x, ae);
+	/* Casting R_xlen_t to size_t is safe. */
+	x_len = (size_t) XLENGTH(x);
+	ae = _new_IntAE(x_len, 0, 0);
+	_IntAE_append(ae, INTEGER(x), x_len);
 	return ae;
 }
 
 IntAE *_new_IntAE_from_CHARACTER(SEXP x, int keyshift)
 {
+	size_t x_len, i;
 	IntAE *ae;
-	int i, *elt;
+	int *elt_p;
 
-	ae = _new_IntAE(LENGTH(x), 0, 0);
-	_IntAE_set_nelt(ae, ae->_buflength);
-	for (i = 0, elt = ae->elts; i < ae->_buflength; i++, elt++) {
-		sscanf(CHAR(STRING_ELT(x, i)), "%d", elt);
-		*elt += keyshift;
+	/* Casting R_xlen_t to size_t is safe. */
+	x_len = (size_t) XLENGTH(x);
+	ae = _new_IntAE(x_len, 0, 0);
+	elt_p = ae->elts;
+	for (i = 0; i < x_len; i++) {
+		sscanf(CHAR(STRING_ELT(x, i)), "%d", elt_p);
+		*(elt_p++) += keyshift;
 	}
+	_IntAE_set_nelt(ae, x_len);
 	return ae;
 }
 
@@ -355,13 +392,17 @@ static int remove_from_IntAE_pool(const IntAE *ae)
 static IntAEAE *IntAEAE_pool[INTAEAE_POOL_MAXLEN];
 static int IntAEAE_pool_len = 0;
 
-int _IntAEAE_get_nelt(const IntAEAE *aeae)
+size_t _IntAEAE_get_nelt(const IntAEAE *aeae)
 {
 	return aeae->_nelt;
 }
 
-int _IntAEAE_set_nelt(IntAEAE *aeae, int nelt)
+size_t _IntAEAE_set_nelt(IntAEAE *aeae, size_t nelt)
 {
+	if (nelt > aeae->_buflength)
+		error("S4Vectors internal error in _IntAEAE_set_nelt(): "
+		      "trying to set a nb of buffer elements that exceeds "
+		      "the buffer length");
 	return aeae->_nelt = nelt;
 }
 
@@ -379,45 +420,54 @@ static IntAEAE *new_empty_IntAEAE()
 	return aeae;
 }
 
-static void IntAEAE_extend(IntAEAE *aeae, int new_buflength)
+static void IntAEAE_extend(IntAEAE *aeae, size_t new_buflength)
 {
-	int old_buflength, i;
+	size_t old_buflength, i;
 
 	old_buflength = aeae->_buflength;
-	if (new_buflength == -1)
-		new_buflength = _get_new_buflength(old_buflength);
-	aeae->elts = (IntAE **) realloc2(aeae->elts, new_buflength,
-					 old_buflength, sizeof(IntAE *));
+	aeae->elts = (IntAE **) realloc2(aeae->elts, old_buflength,
+					 new_buflength, sizeof(IntAE *));
 	for (i = old_buflength; i < new_buflength; i++)
 		aeae->elts[i] = NULL;
 	aeae->_buflength = new_buflength;
 	return;
 }
 
-void _IntAEAE_insert_at(IntAEAE *aeae, int at, IntAE *ae)
+static int IntAEAE_extend_if_full(IntAEAE *aeae)
 {
-	int nelt, i;
+	if (_IntAEAE_get_nelt(aeae) < aeae->_buflength)
+		return 0;
+	IntAEAE_extend(aeae, _increase_buflength(aeae->_buflength));
+	return 1;
+}
+
+void _IntAEAE_insert_at(IntAEAE *aeae, size_t at, IntAE *ae)
+{
+	size_t aeae_nelt, i;
 	IntAE **ae1_p, **ae2_p;
 
-	nelt = _IntAEAE_get_nelt(aeae);
-	if (nelt >= aeae->_buflength)
-		IntAEAE_extend(aeae, -1);
+	aeae_nelt = _IntAEAE_get_nelt(aeae);
+	if (at > aeae_nelt)
+		error("S4Vectors internal error in _IntAEAE_insert_at(): "
+		      "trying to insert a buffer element at an invalid "
+		      "buffer position");
+	IntAEAE_extend_if_full(aeae);
 	if (use_malloc && remove_from_IntAE_pool(ae) == -1)
 		error("S4Vectors internal error in _IntAEAE_insert_at(): "
 		      "IntAE to insert cannot be found in pool for removal");
-	ae1_p = aeae->elts + nelt;
+	ae1_p = aeae->elts + aeae_nelt;
 	ae2_p = ae1_p - 1;
-	for (i = nelt; i > at; i--)
+	for (i = aeae_nelt; i > at; i--)
 		*(ae1_p--) = *(ae2_p--);
 	*ae1_p = ae;
-	_IntAEAE_set_nelt(aeae, nelt + 1);
+	_IntAEAE_set_nelt(aeae, aeae_nelt + 1);
 	return;
 }
 
-IntAEAE *_new_IntAEAE(int buflength, int nelt)
+IntAEAE *_new_IntAEAE(size_t buflength, size_t nelt)
 {
 	IntAEAE *aeae;
-	int i;
+	size_t i;
 	IntAE *ae;
 
 	aeae = new_empty_IntAEAE();
@@ -432,17 +482,21 @@ IntAEAE *_new_IntAEAE(int buflength, int nelt)
 }
 
 /*
- * Left and right IntAEAE objects must have the same length. This is
- * NOT checked!
+ * Parallel append: left and right IntAEAE buffers must have the same length.
  */
-void _IntAEAE_eltwise_append(const IntAEAE *aeae1, const IntAEAE *aeae2)
+void _IntAEAE_pappend(const IntAEAE *aeae1, const IntAEAE *aeae2)
 {
-	int nelt, i;
+	size_t aeae1_nelt, aeae2_nelt, i;
 	IntAE *ae1;
 	const IntAE *ae2;
 
-	nelt = _IntAEAE_get_nelt(aeae1);
-	for (i = 0; i < nelt; i++) {
+	aeae1_nelt = _IntAEAE_get_nelt(aeae1);
+	aeae2_nelt = _IntAEAE_get_nelt(aeae2);
+	if (aeae1_nelt != aeae2_nelt)
+		error("S4Vectors internal error in _IntAEAE_pappend(): "
+		      "the 2 IntAEAE buffers to pappend must have "
+		      "the same length");
+	for (i = 0; i < aeae1_nelt; i++) {
 		ae1 = aeae1->elts[i];
 		ae2 = aeae2->elts[i];
 		_IntAE_append(ae1, ae2->elts, _IntAE_get_nelt(ae2));
@@ -452,30 +506,34 @@ void _IntAEAE_eltwise_append(const IntAEAE *aeae1, const IntAEAE *aeae2)
 
 void _IntAEAE_shift(const IntAEAE *aeae, int shift)
 {
-	int nelt, i;
+	size_t aeae_nelt, i;
 	IntAE *ae;
 
-	nelt = _IntAEAE_get_nelt(aeae);
-	for (i = 0; i < nelt; i++) {
+	aeae_nelt = _IntAEAE_get_nelt(aeae);
+	for (i = 0; i < aeae_nelt; i++) {
 		ae = aeae->elts[i];
-		_IntAE_shift(ae, shift);
+		_IntAE_shift(ae, 0, shift);
 	}
 	return;
 }
 
 /*
- * Left and right IntAEAE objects must have the same length. This is
- * NOT checked!
+ * Left and right IntAEAE buffers must have the same length.
  */
 void _IntAEAE_sum_and_shift(const IntAEAE *aeae1, const IntAEAE *aeae2,
 		int shift)
 {
-	int nelt, i;
+	size_t aeae1_nelt, aeae2_nelt, i;
 	IntAE *ae1;
 	const IntAE *ae2;
 
-	nelt = _IntAEAE_get_nelt(aeae1);
-	for (i = 0; i < nelt; i++) {
+	aeae1_nelt = _IntAEAE_get_nelt(aeae1);
+	aeae2_nelt = _IntAEAE_get_nelt(aeae2);
+	if (aeae1_nelt != aeae2_nelt)
+		error("S4Vectors internal error in _IntAEAE_sum_and_shift(): "
+		      "the 2 IntAEAE buffers to sum_and_shift must have "
+		      "the same length");
+	for (i = 0; i < aeae1_nelt; i++) {
 		ae1 = aeae1->elts[i];
 		ae2 = aeae2->elts[i];
 		_IntAE_sum_and_shift(ae1, ae2, shift);
@@ -489,13 +547,14 @@ void _IntAEAE_sum_and_shift(const IntAEAE *aeae1, const IntAEAE *aeae2,
  */
 SEXP _new_LIST_from_IntAEAE(const IntAEAE *aeae, int mode)
 {
-	int nelt, i;
+	size_t aeae_nelt, i;
 	SEXP ans, ans_elt;
 	const IntAE *ae;
 
-	nelt = _IntAEAE_get_nelt(aeae);
-	PROTECT(ans = NEW_LIST(nelt));
-	for (i = 0; i < nelt; i++) {
+	aeae_nelt = _IntAEAE_get_nelt(aeae);
+	/* ae_nelt <= R_XLEN_T_MAX so casting is safe. */
+	PROTECT(ans = NEW_LIST((R_xlen_t) aeae_nelt));
+	for (i = 0; i < aeae_nelt; i++) {
 		ae = aeae->elts[i];
 		if (_IntAE_get_nelt(ae) != 0 || mode == 0) {
 			PROTECT(ans_elt = _new_INTEGER_from_IntAE(ae));
@@ -516,12 +575,15 @@ SEXP _new_LIST_from_IntAEAE(const IntAEAE *aeae, int mode)
 
 IntAEAE *_new_IntAEAE_from_LIST(SEXP x)
 {
+	size_t x_len;
 	IntAEAE *aeae;
-	int i;
+	size_t i;
 	SEXP x_elt;
 	IntAE *ae;
 
-	aeae = _new_IntAEAE(LENGTH(x), 0);
+	/* Casting R_xlen_t to size_t is safe. */
+	x_len = (size_t) XLENGTH(x);
+	aeae = _new_IntAEAE(x_len, 0);
 	for (i = 0; i < aeae->_buflength; i++) {
 		x_elt = VECTOR_ELT(x, i);
 		if (TYPEOF(x_elt) != INTSXP)
@@ -537,18 +599,18 @@ IntAEAE *_new_IntAEAE_from_LIST(SEXP x)
 
 SEXP _IntAEAE_toEnvir(const IntAEAE *aeae, SEXP envir, int keyshift)
 {
-	int nelt, i;
+	size_t aeae_nelt, i;
 	const IntAE *ae;
 	char key[11];
 	SEXP value;
 
-	nelt = _IntAEAE_get_nelt(aeae);
-	for (i = 0; i < nelt; i++) {
+	aeae_nelt = _IntAEAE_get_nelt(aeae);
+	for (i = 0; i < aeae_nelt; i++) {
 		ae = aeae->elts[i];
 		if (_IntAE_get_nelt(ae) == 0)
 			continue;
 		//snprintf(key, sizeof(key), "%d", i + keyshift);
-		snprintf(key, sizeof(key), "%010d", i + keyshift);
+		snprintf(key, sizeof(key), "%010lu", i + keyshift);
 		PROTECT(value = _new_INTEGER_from_IntAE(ae));
 		defineVar(install(key), value, envir);
 		UNPROTECT(1);
@@ -559,7 +621,7 @@ SEXP _IntAEAE_toEnvir(const IntAEAE *aeae, SEXP envir, int keyshift)
 /* Must be used on a malloc-based IntAEAE */
 static void IntAEAE_free(IntAEAE *aeae)
 {
-	int buflength, i;
+	size_t buflength, i;
 	IntAE *ae;
 
 	buflength = aeae->_buflength;
@@ -595,12 +657,12 @@ static void flush_IntAEAE_pool()
 static IntPairAE *IntPairAE_pool[INTPAIRAE_POOL_MAXLEN];
 static int IntPairAE_pool_len = 0;
 
-int _IntPairAE_get_nelt(const IntPairAE *ae)
+size_t _IntPairAE_get_nelt(const IntPairAE *ae)
 {
 	return _IntAE_get_nelt(ae->a);
 }
 
-int _IntPairAE_set_nelt(IntPairAE *ae, int nelt)
+size_t _IntPairAE_set_nelt(IntPairAE *ae, size_t nelt)
 {
 	_IntAE_set_nelt(ae->a, nelt);
 	_IntAE_set_nelt(ae->b, nelt);
@@ -632,21 +694,21 @@ static IntPairAE *new_empty_IntPairAE()
 	return ae;
 }
 
-static void IntPairAE_extend(IntPairAE *ae, int new_buflength)
+static void IntPairAE_extend(IntPairAE *ae, size_t new_buflength)
 {
 	IntAE_extend(ae->a, new_buflength);
 	IntAE_extend(ae->b, new_buflength);
 	return;
 }
 
-void _IntPairAE_insert_at(IntPairAE *ae, int at, int a, int b)
+void _IntPairAE_insert_at(IntPairAE *ae, size_t at, int a, int b)
 {
 	_IntAE_insert_at(ae->a, at, a);
 	_IntAE_insert_at(ae->b, at, b);
 	return;
 }
 
-IntPairAE *_new_IntPairAE(int buflength, int nelt)
+IntPairAE *_new_IntPairAE(size_t buflength, size_t nelt)
 {
 	IntPairAE *ae;
 
@@ -706,13 +768,17 @@ static int remove_from_IntPairAE_pool(const IntPairAE *ae)
 static IntPairAEAE *IntPairAEAE_pool[INTPAIRAEAE_POOL_MAXLEN];
 static int IntPairAEAE_pool_len = 0;
 
-int _IntPairAEAE_get_nelt(const IntPairAEAE *aeae)
+size_t _IntPairAEAE_get_nelt(const IntPairAEAE *aeae)
 {
 	return aeae->_nelt;
 }
 
-int _IntPairAEAE_set_nelt(IntPairAEAE *aeae, int nelt)
+size_t _IntPairAEAE_set_nelt(IntPairAEAE *aeae, size_t nelt)
 {
+	if (nelt > aeae->_buflength)
+		error("S4Vectors internal error in _IntPairAEAE_set_nelt(): "
+		      "trying to set a nb of buffer elements that exceeds "
+		      "the buffer length");
 	return aeae->_nelt = nelt;
 }
 
@@ -730,46 +796,56 @@ static IntPairAEAE *new_empty_IntPairAEAE()
 	return aeae;
 }
 
-static void IntPairAEAE_extend(IntPairAEAE *aeae, int new_buflength)
+static void IntPairAEAE_extend(IntPairAEAE *aeae, size_t new_buflength)
 {
-	int old_buflength, i;
+	size_t old_buflength, i;
 
 	old_buflength = aeae->_buflength;
-	if (new_buflength == -1)
-		new_buflength = _get_new_buflength(old_buflength);
-	aeae->elts = (IntPairAE **) realloc2(aeae->elts, new_buflength,
-					old_buflength, sizeof(IntPairAE *));
+	aeae->elts = (IntPairAE **) realloc2(aeae->elts, old_buflength,
+					     new_buflength,
+					     sizeof(IntPairAE *));
 	for (i = old_buflength; i < new_buflength; i++)
 		aeae->elts[i] = NULL;
 	aeae->_buflength = new_buflength;
 	return;
 }
 
-void _IntPairAEAE_insert_at(IntPairAEAE *aeae, int at, IntPairAE *ae)
+static int IntPairAEAE_extend_if_full(IntPairAEAE *aeae)
 {
-	int nelt, i;
+	if (_IntPairAEAE_get_nelt(aeae) < aeae->_buflength)
+		return 0;
+	IntPairAEAE_extend(aeae, _increase_buflength(aeae->_buflength));
+	return 1;
+}
+
+void _IntPairAEAE_insert_at(IntPairAEAE *aeae, size_t at, IntPairAE *ae)
+{
+	size_t aeae_nelt, i;
 	IntPairAE **ae1_p, **ae2_p;
 
-	nelt = _IntPairAEAE_get_nelt(aeae);
-	if (nelt >= aeae->_buflength)
-		IntPairAEAE_extend(aeae, -1);
+	aeae_nelt = _IntPairAEAE_get_nelt(aeae);
+	if (at > aeae_nelt)
+		error("S4Vectors internal error in _IntPairAEAE_insert_at(): "
+		      "trying to insert a buffer element at an invalid "
+		      "buffer position");
+	IntPairAEAE_extend_if_full(aeae);
 	if (use_malloc && remove_from_IntPairAE_pool(ae) == -1)
 		error("S4Vectors internal error in _IntPairAEAE_insert_at(): "
 		      "IntPairAE to insert cannot be found in pool for "
 		      "removal");
-	ae1_p = aeae->elts + nelt;
+	ae1_p = aeae->elts + aeae_nelt;
 	ae2_p = ae1_p - 1;
-	for (i = nelt; i > at; i--)
+	for (i = aeae_nelt; i > at; i--)
 		*(ae1_p--) = *(ae2_p--);
 	*ae1_p = ae;
-	_IntPairAEAE_set_nelt(aeae, nelt + 1);
+	_IntPairAEAE_set_nelt(aeae, aeae_nelt + 1);
 	return;
 }
 
-IntPairAEAE *_new_IntPairAEAE(int buflength, int nelt)
+IntPairAEAE *_new_IntPairAEAE(size_t buflength, size_t nelt)
 {
 	IntPairAEAE *aeae;
-	int i;
+	size_t i;
 	IntPairAE *ae;
 
 	aeae = new_empty_IntPairAEAE();
@@ -786,7 +862,7 @@ IntPairAEAE *_new_IntPairAEAE(int buflength, int nelt)
 /* Must be used on a malloc-based IntPairAEAE */
 static void IntPairAEAE_free(IntPairAEAE *aeae)
 {
-	int buflength, i;
+	size_t buflength, i;
 	IntPairAE *ae;
 
 	buflength = aeae->_buflength;
@@ -822,13 +898,17 @@ static void flush_IntPairAEAE_pool()
 static LLongAE *LLongAE_pool[LLONGAE_POOL_MAXLEN];
 static int LLongAE_pool_len = 0;
 
-int _LLongAE_get_nelt(const LLongAE *ae)
+size_t _LLongAE_get_nelt(const LLongAE *ae)
 {
 	return ae->_nelt;
 }
 
-int _LLongAE_set_nelt(LLongAE *ae, int nelt)
+size_t _LLongAE_set_nelt(LLongAE *ae, size_t nelt)
 {
+	if (nelt > ae->_buflength)
+		error("S4Vectors internal error in _LLongAE_set_nelt(): "
+		      "trying to set a nb of buffer elements that exceeds "
+		      "the buffer length");
 	return ae->_nelt = nelt;
 }
 
@@ -848,47 +928,54 @@ static LLongAE *new_empty_LLongAE()
 
 void _LLongAE_set_val(const LLongAE *ae, long long val)
 {
-	int nelt, i;
-	long long *elt;
+	size_t ae_nelt, i;
+	long long *elt_p;
 
-	nelt = _LLongAE_get_nelt(ae);
-	for (i = 0, elt = ae->elts; i < nelt; i++, elt++)
-		*elt = val;
+	ae_nelt = _LLongAE_get_nelt(ae);
+	elt_p = ae->elts;
+	for (i = 0; i < ae_nelt; i++)
+		*(elt_p++) = val;
 	return;
 }
 
-static void LLongAE_extend(LLongAE *ae, int new_buflength)
+static void LLongAE_extend(LLongAE *ae, size_t new_buflength)
 {
-	int old_buflength;
-
-	old_buflength = ae->_buflength;
-	if (new_buflength == -1)
-		new_buflength = _get_new_buflength(old_buflength);
-	ae->elts = (long long *) realloc2(ae->elts, new_buflength,
-					  old_buflength, sizeof(long long));
+	ae->elts = (long long *) realloc2(ae->elts, ae->_buflength,
+					  new_buflength, sizeof(long long));
 	ae->_buflength = new_buflength;
 	return;
 }
 
-void _LLongAE_insert_at(LLongAE *ae, int at, long long val)
+static int LLongAE_extend_if_full(LLongAE *ae)
 {
-	int nelt, i;
+	if (_LLongAE_get_nelt(ae) < ae->_buflength)
+		return 0;
+	LLongAE_extend(ae, _increase_buflength(ae->_buflength));
+	return 1;
+}
+
+void _LLongAE_insert_at(LLongAE *ae, size_t at, long long val)
+{
+	size_t ae_nelt, i;
 	long long *elt1_p;
 	const long long *elt2_p;
 
-	nelt = _LLongAE_get_nelt(ae);
-	if (nelt >= ae->_buflength)
-		LLongAE_extend(ae, -1);
-	elt1_p = ae->elts + nelt;
+	ae_nelt = _LLongAE_get_nelt(ae);
+	if (at > ae_nelt)
+		error("S4Vectors internal error in _LLongAE_insert_at(): "
+		      "trying to insert a buffer element at an invalid "
+		      "buffer position");
+	LLongAE_extend_if_full(ae);
+	elt1_p = ae->elts + ae_nelt;
 	elt2_p = elt1_p - 1;
-	for (i = nelt; i > at; i--)
+	for (i = ae_nelt; i > at; i--)
 		*(elt1_p--) = *(elt2_p--);
 	*elt1_p = val;
-	_LLongAE_set_nelt(ae, nelt + 1);
+	_LLongAE_set_nelt(ae, ae_nelt + 1);
 	return;
 }
 
-LLongAE *_new_LLongAE(int buflength, int nelt, long long val)
+LLongAE *_new_LLongAE(size_t buflength, size_t nelt, long long val)
 {
 	LLongAE *ae;
 
@@ -931,13 +1018,17 @@ static void flush_LLongAE_pool()
 static CharAE *CharAE_pool[CHARAE_POOL_MAXLEN];
 static int CharAE_pool_len = 0;
 
-int _CharAE_get_nelt(const CharAE *ae)
+size_t _CharAE_get_nelt(const CharAE *ae)
 {
 	return ae->_nelt;
 }
 
-int _CharAE_set_nelt(CharAE *ae, int nelt)
+size_t _CharAE_set_nelt(CharAE *ae, size_t nelt)
 {
+	if (nelt > ae->_buflength)
+		error("S4Vectors internal error in _CharAE_set_nelt(): "
+		      "trying to set a nb of buffer elements that exceeds "
+		      "the buffer length");
 	return ae->_nelt = nelt;
 }
 
@@ -955,38 +1046,44 @@ static CharAE *new_empty_CharAE()
 	return ae;
 }
 
-static void CharAE_extend(CharAE *ae, int new_buflength)
+static void CharAE_extend(CharAE *ae, size_t new_buflength)
 {
-	int old_buflength;
-
-	old_buflength = ae->_buflength;
-	if (new_buflength == -1)
-		new_buflength = _get_new_buflength(old_buflength);
-	ae->elts = (char *) realloc2(ae->elts, new_buflength,
-				     old_buflength, sizeof(char));
+	ae->elts = (char *) realloc2(ae->elts, ae->_buflength,
+				     new_buflength, sizeof(char));
 	ae->_buflength = new_buflength;
 	return;
 }
 
-void _CharAE_insert_at(CharAE *ae, int at, char c)
+static int CharAE_extend_if_full(CharAE *ae)
 {
-	int nelt, i;
+	if (_CharAE_get_nelt(ae) < ae->_buflength)
+		return 0;
+	CharAE_extend(ae, _increase_buflength(ae->_buflength));
+	return 1;
+}
+
+void _CharAE_insert_at(CharAE *ae, size_t at, char c)
+{
+	size_t ae_nelt, i;
 	char *elt1_p;
 	const char *elt2_p;
 
-	nelt = _CharAE_get_nelt(ae);
-	if (nelt >= ae->_buflength)
-		CharAE_extend(ae, -1);
-	elt1_p = ae->elts + nelt;
+	ae_nelt = _CharAE_get_nelt(ae);
+	if (at > ae_nelt)
+		error("S4Vectors internal error in _CharAE_insert_at(): "
+		      "trying to insert a buffer element at an invalid "
+		      "buffer position");
+	CharAE_extend_if_full(ae);
+	elt1_p = ae->elts + ae_nelt;
 	elt2_p = elt1_p - 1;
-	for (i = nelt; i > at; i--)
+	for (i = ae_nelt; i > at; i--)
 		*(elt1_p--) = *(elt2_p--);
 	*elt1_p = c;
-	_CharAE_set_nelt(ae, nelt + 1);
+	_CharAE_set_nelt(ae, ae_nelt + 1);
 	return;
 }
 
-CharAE *_new_CharAE(int buflength)
+CharAE *_new_CharAE(size_t buflength)
 {
 	CharAE *ae;
 
@@ -1006,17 +1103,17 @@ CharAE *_new_CharAE_from_string(const char *string)
 	return ae;
 }
 
-void _append_string_to_CharAE(CharAE *ae, const char *string)
+void _CharAE_append_string(CharAE *ae, const char *string)
 {
-	int nnewval, nelt, new_nelt;
+	size_t nnewval, ae_nelt, new_nelt;
 	char *dest;
 
 	nnewval = strlen(string);
-	nelt = _CharAE_get_nelt(ae);
-	new_nelt = nelt + nnewval;
+	ae_nelt = _CharAE_get_nelt(ae);
+	new_nelt = ae_nelt + nnewval;
 	if (new_nelt > ae->_buflength)
 		CharAE_extend(ae, new_nelt);
-	dest = ae->elts + nelt;
+	dest = ae->elts + ae_nelt;
 	memcpy(dest, string, sizeof(char) * nnewval);
 	_CharAE_set_nelt(ae, new_nelt);
 	return;
@@ -1024,36 +1121,37 @@ void _append_string_to_CharAE(CharAE *ae, const char *string)
 
 /*
  * Delete 'nelt' elements, starting at position 'at'.
- * Doing _CharAE_delete_at(x, at, nelt) is equivalent to doing
- * _CharAE_delete_at(x, at, 1) 'nelt' times.
+ * Calling _CharAE_delete_at(x, at, nelt) is equivalent to calling
+ * _CharAE_delete_at(x, at, 1) nelt times.
  */
-void _CharAE_delete_at(CharAE *ae, int at, int nelt)
+void _CharAE_delete_at(CharAE *ae, size_t at, size_t nelt)
 {
 	char *c1_p;
 	const char *c2_p;
-	int nelt0, i2;
+	size_t ae_nelt, i2;
 
 	if (nelt == 0)
 		return;
 	c1_p = ae->elts + at;
 	c2_p = c1_p + nelt;
-	nelt0 = _CharAE_get_nelt(ae);
-	for (i2 = at + nelt; i2 < nelt0; i2++)
+	ae_nelt = _CharAE_get_nelt(ae);
+	for (i2 = at + nelt; i2 < ae_nelt; i2++)
 		*(c1_p++) = *(c2_p++);
-	_CharAE_set_nelt(ae, nelt0 - nelt);
+	_CharAE_set_nelt(ae, ae_nelt - nelt);
 	return;
 }
 
 SEXP _new_RAW_from_CharAE(const CharAE *ae)
 {
-	int nelt;
+	size_t ae_nelt;
 	SEXP ans;
 
 	if (sizeof(Rbyte) != sizeof(char)) // should never happen!
 		error("_new_RAW_from_CharAE(): sizeof(Rbyte) != sizeof(char)");
-	nelt = _CharAE_get_nelt(ae);
-	PROTECT(ans = NEW_RAW(nelt));
-	memcpy(RAW(ans), ae->elts, sizeof(char) * nelt);
+	ae_nelt = _CharAE_get_nelt(ae);
+	/* ae_nelt <= R_XLEN_T_MAX so casting is safe. */
+	PROTECT(ans = NEW_RAW((R_xlen_t) ae_nelt));
+	memcpy(RAW(ans), ae->elts, ae_nelt * sizeof(char));
 	UNPROTECT(1);
 	return ans;
 }
@@ -1061,18 +1159,16 @@ SEXP _new_RAW_from_CharAE(const CharAE *ae)
 /* only until we have a bitset or something smaller than char */
 SEXP _new_LOGICAL_from_CharAE(const CharAE *ae)
 {
-	int nelt, i, *ans_elt;
+	size_t ae_nelt, i;
 	SEXP ans;
-	const char *elt;
+	const char *elt_p;
 
-	nelt = _CharAE_get_nelt(ae);
-	PROTECT(ans = NEW_LOGICAL(nelt));
-	for (i = 0, ans_elt = LOGICAL(ans), elt = ae->elts;
-	     i < nelt;
-	     i++, ans_elt++, elt++)
-	{
-		*ans_elt = *elt;
-	}
+	ae_nelt = _CharAE_get_nelt(ae);
+	/* ae_nelt <= R_XLEN_T_MAX so casting is safe. */
+	PROTECT(ans = NEW_LOGICAL((R_xlen_t) ae_nelt));
+	elt_p = ae->elts;
+	for (i = 0; i < ae_nelt; i++)
+		LOGICAL(ans)[i] = *(elt_p++);
 	UNPROTECT(1);
 	return ans;
 }
@@ -1124,13 +1220,17 @@ static int remove_from_CharAE_pool(const CharAE *ae)
 static CharAEAE *CharAEAE_pool[CHARAEAE_POOL_MAXLEN];
 static int CharAEAE_pool_len = 0;
 
-int _CharAEAE_get_nelt(const CharAEAE *aeae)
+size_t _CharAEAE_get_nelt(const CharAEAE *aeae)
 {
 	return aeae->_nelt;
 }
 
-int _CharAEAE_set_nelt(CharAEAE *aeae, int nelt)
+size_t _CharAEAE_set_nelt(CharAEAE *aeae, size_t nelt)
 {
+	if (nelt > aeae->_buflength)
+		error("S4Vectors internal error in _CharAEAE_set_nelt(): "
+		      "trying to set a nb of buffer elements that exceeds "
+		      "the buffer length");
 	return aeae->_nelt = nelt;
 }
 
@@ -1148,45 +1248,54 @@ static CharAEAE *new_empty_CharAEAE()
 	return aeae;
 }
 
-static void CharAEAE_extend(CharAEAE *aeae, int new_buflength)
+static void CharAEAE_extend(CharAEAE *aeae, size_t new_buflength)
 {
-	int old_buflength, i;
+	size_t old_buflength, i;
 
 	old_buflength = aeae->_buflength;
-	if (new_buflength == -1)
-		new_buflength = _get_new_buflength(old_buflength);
-	aeae->elts = (CharAE **) realloc2(aeae->elts, new_buflength,
-					  old_buflength, sizeof(CharAE *));
+	aeae->elts = (CharAE **) realloc2(aeae->elts, old_buflength,
+					  new_buflength, sizeof(CharAE *));
 	for (i = old_buflength; i < new_buflength; i++)
 		aeae->elts[i] = NULL;
 	aeae->_buflength = new_buflength;
 	return;
 }
 
-void _CharAEAE_insert_at(CharAEAE *aeae, int at, CharAE *ae)
+static int CharAEAE_extend_if_full(CharAEAE *aeae)
 {
-	int nelt, i;
+	if (_CharAEAE_get_nelt(aeae) < aeae->_buflength)
+		return 0;
+	CharAEAE_extend(aeae, _increase_buflength(aeae->_buflength));
+	return 1;
+}
+
+void _CharAEAE_insert_at(CharAEAE *aeae, size_t at, CharAE *ae)
+{
+	size_t aeae_nelt, i;
 	CharAE **ae1_p, **ae2_p;
 
-	nelt = _CharAEAE_get_nelt(aeae);
-	if (nelt >= aeae->_buflength)
-		CharAEAE_extend(aeae, -1);
+	aeae_nelt = _CharAEAE_get_nelt(aeae);
+	if (at > aeae_nelt)
+		error("S4Vectors internal error in _CharAEAE_insert_at(): "
+		      "trying to insert a buffer element at an invalid "
+		      "buffer position");
+	CharAEAE_extend_if_full(aeae);
 	if (use_malloc && remove_from_CharAE_pool(ae) == -1)
 		error("S4Vectors internal error in _CharAEAE_insert_at(): "
 		      "CharAE to insert cannot be found in pool for removal");
-	ae1_p = aeae->elts + nelt;
+	ae1_p = aeae->elts + aeae_nelt;
 	ae2_p = ae1_p - 1;
-	for (i = nelt; i > at; i--)
+	for (i = aeae_nelt; i > at; i--)
 		*(ae1_p--) = *(ae2_p--);
 	*ae1_p = ae;
-	_CharAEAE_set_nelt(aeae, nelt + 1);
+	_CharAEAE_set_nelt(aeae, aeae_nelt + 1);
 	return;
 }
 
-CharAEAE *_new_CharAEAE(int buflength, int nelt)
+CharAEAE *_new_CharAEAE(size_t buflength, size_t nelt)
 {
 	CharAEAE *aeae;
-	int i;
+	size_t i;
 	CharAE *ae;
 
 	aeae = new_empty_CharAEAE();
@@ -1200,7 +1309,7 @@ CharAEAE *_new_CharAEAE(int buflength, int nelt)
 	return aeae;
 }
 
-void _append_string_to_CharAEAE(CharAEAE *aeae, const char *string)
+void _CharAEAE_append_string(CharAEAE *aeae, const char *string)
 {
 	CharAE *ae;
 
@@ -1211,15 +1320,21 @@ void _append_string_to_CharAEAE(CharAEAE *aeae, const char *string)
 
 SEXP _new_CHARACTER_from_CharAEAE(const CharAEAE *aeae)
 {
-	int nelt, i;
+	size_t aeae_nelt, i, ae_nelt;
 	SEXP ans, ans_elt;
 	CharAE *ae;
 
-	nelt = _CharAEAE_get_nelt(aeae);
-	PROTECT(ans = NEW_CHARACTER(nelt));
-	for (i = 0; i < nelt; i++) {
+	aeae_nelt = _CharAEAE_get_nelt(aeae);
+	/* ae_nelt <= R_XLEN_T_MAX so casting is safe. */
+	PROTECT(ans = NEW_CHARACTER((R_xlen_t) aeae_nelt));
+	for (i = 0; i < aeae_nelt; i++) {
 		ae = aeae->elts[i];
-		PROTECT(ans_elt = mkCharLen(ae->elts, _CharAE_get_nelt(ae)));
+		ae_nelt = _CharAE_get_nelt(ae);
+		if (ae_nelt > INT_MAX)
+			error("S4Vectors internal error in "
+			      "_new_CHARACTER_from_CharAEAE: character "
+			      "buffer is too long for mkCharLen()");
+		PROTECT(ans_elt = mkCharLen(ae->elts, (int) ae_nelt));
 		SET_STRING_ELT(ans, i, ans_elt);
 		UNPROTECT(1);
 	}
@@ -1230,7 +1345,7 @@ SEXP _new_CHARACTER_from_CharAEAE(const CharAEAE *aeae)
 /* Must be used on a malloc-based CharAEAE */
 static void CharAEAE_free(CharAEAE *aeae)
 {
-	int buflength, i;
+	size_t buflength, i;
 	CharAE *ae;
 
 	buflength = aeae->_buflength;
