@@ -19,17 +19,42 @@ int _safe_int_add(
 	int y
 );
 
+int _safe_int_subtract(
+	int x,
+	int y
+);
+
 int _safe_int_mult(
 	int x,
 	int y
 );
 
+long long int _safe_llint_add(
+	long long int x,
+	long long int y
+);
+
+long long int _safe_llint_subtract(
+	long long int x,
+	long long int y
+);
+
+long long int _safe_llint_mult(
+	long long int x,
+	long long int y
+);
+
 
 /* sort_utils.c */
 
+SEXP test_sort_ushort_array(
+	SEXP x,
+	SEXP desc
+);
+
 void _sort_int_array(
 	int *x,
-	int nelt,
+	size_t nelt,
 	int desc
 );
 
@@ -186,13 +211,13 @@ void _set_hbucket_val(
 
 SEXP AEbufs_use_malloc(SEXP x);
 
-int _get_new_buflength(int buflength);
+size_t _increase_buflength(size_t buflength);
 
-int _IntAE_get_nelt(const IntAE *ae);
+size_t _IntAE_get_nelt(const IntAE *ae);
 
-int _IntAE_set_nelt(
+size_t _IntAE_set_nelt(
 	IntAE *ae,
-	int nelt
+	size_t nelt
 );
 
 void _IntAE_set_val(
@@ -202,29 +227,31 @@ void _IntAE_set_val(
 
 void _IntAE_insert_at(
 	IntAE *ae,
-	int at,
+	size_t at,
 	int val
 );
 
 IntAE *_new_IntAE(
-	int buflength,
-	int nelt,
+	size_t buflength,
+	size_t nelt,
 	int val
 );
 
 void _IntAE_append(
 	IntAE *ae,
 	const int *newvals,
-	int nnewval
+	size_t nnewval
 );
 
 void _IntAE_delete_at(
 	IntAE *ae,
-	int at
+	size_t at,
+	size_t nelt
 );
 
 void _IntAE_shift(
 	const IntAE *ae,
+	size_t offset,
 	int shift
 );
 
@@ -234,19 +261,16 @@ void _IntAE_sum_and_shift(
 	int shift
 );
 
-void _IntAE_append_shifted_vals(
-	IntAE *ae,
-	const int *newvals,
-	int nnewval,
-	int shift
-);
-
 void _IntAE_qsort(
 	const IntAE *ae,
+	size_t offset,
 	int desc
 );
 
-void _IntAE_delete_adjdups(IntAE *ae);
+void _IntAE_uniq(
+	IntAE *ae,
+	size_t offset
+);
 
 SEXP _new_INTEGER_from_IntAE(const IntAE *ae);
 
@@ -257,25 +281,25 @@ IntAE *_new_IntAE_from_CHARACTER(
 	int keyshift
 );
 
-int _IntAEAE_get_nelt(const IntAEAE *aeae);
+size_t _IntAEAE_get_nelt(const IntAEAE *aeae);
 
-int _IntAEAE_set_nelt(
+size_t _IntAEAE_set_nelt(
 	IntAEAE *aeae,
-	int nelt
+	size_t nelt
 );
 
 void _IntAEAE_insert_at(
 	IntAEAE *aeae,
-	int at,
+	size_t at,
 	IntAE *ae
 );
 
 IntAEAE *_new_IntAEAE(
-	int buflength,
-	int nelt
+	size_t buflength,
+	size_t nelt
 );
 
-void _IntAEAE_eltwise_append(
+void _IntAEAE_pappend(
 	const IntAEAE *aeae1,
 	const IntAEAE *aeae2
 );
@@ -304,48 +328,48 @@ SEXP _IntAEAE_toEnvir(
 	int keyshift
 );
 
-int _IntPairAE_get_nelt(const IntPairAE *ae);
+size_t _IntPairAE_get_nelt(const IntPairAE *ae);
 
-int _IntPairAE_set_nelt(
+size_t _IntPairAE_set_nelt(
 	IntPairAE *ae,
-	int nelt
+	size_t nelt
 );
 
 void _IntPairAE_insert_at(
 	IntPairAE *ae,
-	int at,
+	size_t at,
 	int a,
 	int b
 );
 
 IntPairAE *_new_IntPairAE(
-	int buflength,
-	int nelt
+	size_t buflength,
+	size_t nelt
 );
 
-int _IntPairAEAE_get_nelt(const IntPairAEAE *aeae);
+size_t _IntPairAEAE_get_nelt(const IntPairAEAE *aeae);
 
-int _IntPairAEAE_set_nelt(
+size_t _IntPairAEAE_set_nelt(
 	IntPairAEAE *aeae,
-	int nelt
+	size_t nelt
 );
 
 void _IntPairAEAE_insert_at(
 	IntPairAEAE *aeae,
-	int at,
+	size_t at,
 	IntPairAE *ae
 );
 
 IntPairAEAE *_new_IntPairAEAE(
-	int buflength,
-	int nelt
+	size_t buflength,
+	size_t nelt
 );
 
-int _LLongAE_get_nelt(const LLongAE *ae);
+size_t _LLongAE_get_nelt(const LLongAE *ae);
 
-int _LLongAE_set_nelt(
+size_t _LLongAE_set_nelt(
 	LLongAE *ae,
-	int nelt
+	size_t nelt
 );
 
 void _LLongAE_set_val(
@@ -355,67 +379,67 @@ void _LLongAE_set_val(
 
 void _LLongAE_insert_at(
 	LLongAE *ae,
-	int at,
+	size_t at,
 	long long val
 );
 
 LLongAE *_new_LLongAE(
-	int buflength,
-	int nelt,
+	size_t buflength,
+	size_t nelt,
 	long long val
 );
 
-int _CharAE_get_nelt(const CharAE *ae);
+size_t _CharAE_get_nelt(const CharAE *ae);
 
-int _CharAE_set_nelt(
+size_t _CharAE_set_nelt(
 	CharAE *ae,
-	int nelt
+	size_t nelt
 );
 
 void _CharAE_insert_at(
 	CharAE *ae,
-	int at,
+	size_t at,
 	char c
 );
 
-CharAE *_new_CharAE(int buflength);
+CharAE *_new_CharAE(size_t buflength);
 
 CharAE *_new_CharAE_from_string(const char *string);
 
-void _append_string_to_CharAE(
+void _CharAE_append_string(
 	CharAE *ae,
 	const char *string
 );
 
 void _CharAE_delete_at(
 	CharAE *ae,
-	int at,
-	int nelt
+	size_t at,
+	size_t nelt
 );
 
 SEXP _new_RAW_from_CharAE(const CharAE *ae);
 
 SEXP _new_LOGICAL_from_CharAE(const CharAE *ae);
 
-int _CharAEAE_get_nelt(const CharAEAE *aeae);
+size_t _CharAEAE_get_nelt(const CharAEAE *aeae);
 
-int _CharAEAE_set_nelt(
+size_t _CharAEAE_set_nelt(
 	CharAEAE *aeae,
-	int nelt
+	size_t nelt
 );
 
 void _CharAEAE_insert_at(
 	CharAEAE *aeae,
-	int at,
+	size_t at,
 	CharAE *ae
 );
 
 CharAEAE *_new_CharAEAE(
-	int buflength,
-	int nelt
+	size_t buflength,
+	size_t nelt
 );
 
-void _append_string_to_CharAEAE(
+void _CharAEAE_append_string(
 	CharAEAE *aeae,
 	const char *string
 );
@@ -435,15 +459,36 @@ const char *_get_classname(SEXP x);
 SEXP anyMissing(SEXP x);
 
 
-/* vector_utils.c */
+/* Linteger_class.c */
 
-int _vector_memcmp(
-	SEXP x1,
-	int x1_offset,
-	SEXP x2,
-	int x2_offset,
-	int nelt
-);
+int _is_Linteger(SEXP x);
+
+SEXP make_RAW_from_NA_LINTEGER();
+
+R_xlen_t _get_Linteger_length(SEXP x);
+
+long long int *_get_Linteger_dataptr(SEXP x);
+
+SEXP _alloc_Linteger(const char *classname, R_xlen_t length);
+
+SEXP new_Linteger_from_LOGICAL(SEXP x);
+
+SEXP new_Linteger_from_INTEGER(SEXP x);
+
+SEXP new_Linteger_from_NUMERIC(SEXP x);
+
+SEXP new_Linteger_from_CHARACTER(SEXP x);
+
+SEXP new_LOGICAL_from_Linteger(SEXP x);
+
+SEXP new_INTEGER_from_Linteger(SEXP x);
+
+SEXP new_NUMERIC_from_Linteger(SEXP x);
+
+SEXP new_CHARACTER_from_Linteger(SEXP x);
+
+
+/* subsetting_utils.c */
 
 int _copy_vector_block(
 	SEXP dest,
@@ -453,6 +498,14 @@ int _copy_vector_block(
 	int block_width
 );
 
+int _copy_vector_positions(
+	SEXP dest,
+	int dest_offset,
+	SEXP src,
+	const int *pos,
+	int npos
+);
+
 int _copy_vector_ranges(
 	SEXP dest,
 	int dest_offset,
@@ -462,19 +515,41 @@ int _copy_vector_ranges(
 	int nranges
 );
 
-SEXP _subset_vectorORfactor_by_ranges(
+SEXP _subset_vector_OR_factor_by_positions(
+	SEXP x,
+	const int *pos,
+	int npos
+);
+
+SEXP _subset_vector_OR_factor_by_ranges(
 	SEXP x,
 	const int *start,
 	const int *width,
 	int nranges
 );
 
-SEXP vectorORfactor_extract_ranges(
+SEXP vector_OR_factor_extract_positions(
+	SEXP x,
+	SEXP pos
+);
+
+SEXP vector_OR_factor_extract_ranges(
 	SEXP x,
 	SEXP start,
 	SEXP width
 );
 
+
+/* vector_utils.c */
+
+int _vector_memcmp(
+	SEXP x1,
+	int x1_offset,
+	SEXP x2,
+	int x2_offset,
+	int nelt
+);
+
 SEXP sapply_NROW(SEXP x);
 
 SEXP _list_as_data_frame(
@@ -502,14 +577,6 @@ SEXP compact_bitvector_set_op(SEXP query, SEXP ref, SEXP align);
 
 SEXP Integer_any_missing_or_outside(SEXP x, SEXP lower, SEXP upper);
 
-int _sum_non_neg_ints(
-	const int *x,
-	int x_len,
-	const char *varname
-);
-
-SEXP Integer_sum_non_neg_vals(SEXP x);
-
 SEXP Integer_diff_with_0(SEXP x);
 
 SEXP Integer_diff_with_last(SEXP x, SEXP last);
@@ -706,6 +773,62 @@ SEXP top_prenv(SEXP nm, SEXP env);
 SEXP top_prenv_dots(SEXP env);
 
 
+/* map_ranges_to_runs.c */
+
+const char *_simple_range_mapper(
+	const int *run_lengths,
+	int nrun,
+	int range_start,
+	int range_end,
+	int *mapped_range_offset,
+	int *mapped_range_span,
+	int *mapped_range_Ltrim,
+	int *mapped_range_Rtrim
+);
+
+const char *_simple_position_mapper(
+	const int *run_lengths,
+	int nrun,
+	int pos,
+	int *mapped_pos
+);
+
+const char *_ranges_mapper(
+	const int *run_lengths,
+	int nrun,
+	const int *start,
+	const int *width,
+	int nranges,
+	int *mapped_range_offset,
+	int *mapped_range_span,
+	int *mapped_range_Ltrim,
+	int *mapped_range_Rtrim,
+	int method
+);
+
+const char *_positions_mapper(
+	const int *run_lengths,
+	int nrun,
+	const int *pos,
+	int npos,
+	int *mapped_pos,
+	int method
+);
+
+SEXP map_ranges(
+	SEXP run_lengths,
+	SEXP start,
+	SEXP width,
+	SEXP method
+);
+
+SEXP map_positions(
+	SEXP run_lengths,
+	SEXP pos,
+	SEXP method
+);
+
+
 /* Hits_class.c */
 
 SEXP _new_Hits(
@@ -743,77 +866,66 @@ SEXP make_all_group_inner_hits(
 
 /* Rle_class.c */
 
+SEXP Rle_length(SEXP x);
+
+SEXP Rle_valid(SEXP x);
+
 SEXP _construct_logical_Rle(
-	const int *values,
-	int nvalues,
-	const int *lengths,
-	int buflength
+	R_xlen_t nrun_in,
+	const int *values_in,
+	const void *lengths_in,
+	int lengths_in_is_L
 );
 
 SEXP _construct_integer_Rle(
-	const int *values,
-	int nvalues,
-	const int *lengths,
-	int buflength
+	R_xlen_t nrun_in,
+	const int *values_in,
+	const void *lengths_in,
+	int lengths_in_is_L
 );
 
 SEXP _construct_numeric_Rle(
-	const double *values,
-	int nvalues,
-	const int *lengths,
-	int buflength
+	R_xlen_t nrun_in,
+	const double *values_in,
+	const void *lengths_in,
+	int lengths_in_is_L
 );
 
 SEXP _construct_complex_Rle(
-	const Rcomplex *values,
-	int nvalues,
-	const int *lengths,
-	int buflength
+	R_xlen_t nrun_in,
+	const Rcomplex *values_in,
+	const void *lengths_in,
+	int lengths_in_is_L
 );
 
 SEXP _construct_character_Rle(
-	SEXP values,
-	const int *lengths,
-	int buflength
+	SEXP values_in,
+	const void *lengths_in,
+	int lengths_in_is_L
 );
 
 SEXP _construct_raw_Rle(
-	const Rbyte *values,
-	int nvalues,
-	const int *lengths,
-	int buflength
+	R_xlen_t nrun_in,
+	const Rbyte *values_in,
+	const void *lengths_in,
+	int lengths_in_is_L
 );
 
 SEXP _construct_Rle(
-	SEXP values,
-	const int *lengths,
-	int buflength
+	SEXP values_in,
+	const void *lengths_in,
+	int lengths_in_is_L
 );
 
 SEXP Rle_constructor(
-	SEXP values,
-	SEXP lengths,
-	SEXP check,
-	SEXP buflength
+	SEXP values_in,
+	SEXP lengths_in
 );
 
 SEXP Rle_start(SEXP x);
 
 SEXP Rle_end(SEXP x);
 
-SEXP ranges_to_runs_mapper(
-	SEXP run_lengths,
-	SEXP start,
-	SEXP width,
-	SEXP method
-);
-
-SEXP Rle_extract_range(
-	SEXP x,
-	SEXP start,
-	SEXP end
-);
-
 SEXP _subset_Rle_by_ranges(
 	SEXP x,
 	const int *start,
@@ -823,6 +935,19 @@ SEXP _subset_Rle_by_ranges(
 	int as_list
 );
 
+SEXP _subset_Rle_by_positions(
+	SEXP x,
+	const int *pos,
+	int npos,
+	int method
+);
+
+SEXP Rle_extract_range(
+	SEXP x,
+	SEXP start,
+	SEXP end
+);
+
 SEXP Rle_extract_ranges(
 	SEXP x,
 	SEXP start,
@@ -831,6 +956,12 @@ SEXP Rle_extract_ranges(
 	SEXP as_list
 );
 
+SEXP Rle_extract_positions(
+	SEXP x,
+	SEXP pos,
+	SEXP method
+);
+
 SEXP Rle_getStartEndRunAndOffset(
 	SEXP x,
 	SEXP start,
